/* global chrome */

const COCHAV_HANOFESH_URL = 'https://cochav-hanofesh.com/?source=chrome_ext';
const DEFAULT_CONTEXT_MENU_ITEMS = [
  'email@gmail.com'
];
const DEFAULT_EMAIL_SUFFIXES = [
  'gmail.com',
  'outlook.com',
  'hotmail.com',
  'okmail.co.il',
  'enativ.com',
  'walla.co.il',
  'yahoo.com',
  'netvision.net.il',
  'bezeqint.net',
  '012.net.il'
];

function loadOptions () {
  chrome.storage.sync.get('options', (storage) => {
    const contextMenuItems = (storage.options && storage.options.pasteEmailList
      ? storage.options.pasteEmailList
      : []);
    setUpContextMenus(contextMenuItems);
  });
}
loadOptions();

function setUpContextMenus (contextMenuItems) {
  if (!contextMenuItems || !contextMenuItems.length) {
    setUpEmptyContextMenu();
    return;
  }

  contextMenuItems.forEach((command) => {
    chrome.contextMenus.create({
      title: command,
      id: command,
      contexts: ['editable'],
      onclick: sendMessage
    });
  });

  chrome.contextMenus.create({ type: 'separator', contexts: ['editable'] });

  chrome.contextMenus.create({
    title: 'כוכב הנופש',
    id: 'כוכב הנופש',
    contexts: ['editable'],
    onclick: openCochavHanofeshPage
  });
}

function setUpEmptyContextMenu () {
  chrome.contextMenus.create({ type: 'separator', contexts: ['editable'] });
  chrome.contextMenus.create({
    title: 'ריק',
    id: 'empty פון',
    enabled: false,
    contexts: ['editable']
  });
}

function saveDefaultOptions () {
  return new Promise((resolve, reject) => {
    chrome.storage.sync.set({
      options: {
        pasteEmailList: DEFAULT_CONTEXT_MENU_ITEMS,
        autocompleteEmailList: DEFAULT_EMAIL_SUFFIXES
      }
    }, resolve);
  });
}

chrome.runtime.onInstalled.addListener(() => {
  saveDefaultOptions()
    .then(() => {
      chrome.contextMenus.removeAll();
      loadOptions();
    });
});

chrome.storage.onChanged.addListener((changes, namespace) => {
  if (changes.options.newValue && changes.options.newValue.pasteEmailList) {
    chrome.contextMenus.removeAll();
    setUpContextMenus(changes.options.newValue.pasteEmailList);
  }
});

function sendMessage (itemData) {
  chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
    const tabId = tabs[0].id;
    chrome.tabs.sendMessage(tabId, {
      action: 'paste',
      value: itemData
    });
  });
}

function openCochavHanofeshPage () {
  chrome.tabs.create(
    { url: COCHAV_HANOFESH_URL, active: true }
  );
}
