/* global chrome */
const COCHAV_HANOFESH_URL = 'https://cochav-hanofesh.com/?source=chrome_ext';

/** Email autocomplete; **/
let suffixList;
chrome.storage.sync.get('options', (storage) => {
  suffixList = (storage.options && storage.options.autocompleteEmailList
    ? storage.options.autocompleteEmailList
    : []);
});

$(document.body).append('<div class="email-autocomplete-ext"></div>');

const autocompleteElement = $('.email-autocomplete-ext');
let isShown = false;

function showAutocompleteElement (value, position, targetField) {
  if (isShown) {
    return;
  }

  $(suffixList).each((index, suffix) => {
    autocompleteElement.append(`<div class="email-complete-value" tabindex="1" data-suffix="${suffix}"><span>${value}</span><span class="suffix">@${suffix}</span></div>`);
  });
  autocompleteElement.append('<div class="email-complete-credit" tabindex="1"><a>כוכב הנופש</a></div>');
  autocompleteElement.show()
    .css({
      top: position.top || 'auto',
      right: position.right || 'auto',
      left: position.left || 'auto'
    });
  isShown = true;

  setupAutocompleteListeners(targetField);
}

function setupAutocompleteListeners (targetField) {
  $('.email-autocomplete-ext .email-complete-value').mousedown((event) => {
    completeEmail(targetField, $(event.currentTarget).attr('data-suffix'));
  });

  $('.email-autocomplete-ext .email-complete-credit').click((event) => {
    const win = window.open(COCHAV_HANOFESH_URL, '_blank');
    win.focus();
  });

  $('.email-autocomplete-ext .email-complete-value').keydown((event) => {
    event.stopPropagation();
    if ($(event.target).keyCode === 13) {
      $(event.target).trigger('mousedown');
    }
  });

  $(document).click(hideAutocompleteElement);
  $(document).keydown(navigationListener);
}

function navigationListener (event) {
  event.stopPropagation();
  let action;
  let childToCheck;
  let childToSelect;
  let focusedValue = $('.email-autocomplete-ext > .focus');

  if (event.keyCode === 13) {
    focusedValue.trigger('mousedown');
    focusedValue.trigger('click');
    return false;
  } else if (event.keyCode === 40) {
    action = 'next';
    childToCheck = 'last';
    childToSelect = 'first';
  } else if (event.keyCode === 38) {
    action = 'prev';
    childToCheck = 'first';
    childToSelect = 'last';
  }
  if (!action) {
    return;
  }

  $('.email-autocomplete-ext > *').removeClass('focus');
  if (!focusedValue.length || focusedValue.is(':' + childToCheck + '-child')) {
    focusedValue = $('.email-autocomplete-ext > *')[childToSelect]();
  } else {
    focusedValue = focusedValue[action]();
  }
  focusedValue.addClass('focus');
}

function hideAutocompleteElement () {
  autocompleteElement.hide().html('');
  $(document).off('click', hideAutocompleteElement);
  $(document).off('keydown', navigationListener);
  isShown = false;
}

function lastCharIsAt (event, targetField, targetFieldValue) {
  const tag = targetField[0].tagName.toLowerCase();

  if ((tag !== 'input' && tag !== 'textarea') ||
    targetField[0].type === 'password' ||
    $(targetField[0]).css('-webkit-text-security') === 'disc' ||
    (event.originalEvent.data !== '@' && targetFieldValue[targetFieldValue.length - 1] !== '@') ||
    (targetField[0].selectionEnd && targetField[0].selectionEnd < targetFieldValue.length)) {
    return false;
  }

  const emailValue = targetFieldValue.split(' ')[targetFieldValue.split(' ').length - 1].slice(0, -1) || '';

  if (!emailValue.trim().length ||
    emailValue.includes('@')) {
    return false;
  }

  return true;
}

function completeEmail (inputElement, value) {
  $(inputElement).trigger('focus');
  const valueToPaste = $(inputElement).val() + value;
  $(inputElement).val(valueToPaste);
  $(inputElement)[0].dispatchEvent(new Event('input', {
    bubbles: true,
    cancelable: true
  }));

  setTimeout(() => {
    $(inputElement).trigger('focus');
  });
  hideAutocompleteElement();
}

$(document).on('input', (event) => {
  if (!suffixList.length) {
    return;
  }

  const targetField = $(event.target);
  const targetFieldValue = targetField.val();

  if (lastCharIsAt(event, targetField, targetFieldValue)) {
    const emailValue = targetFieldValue.split(' ')[targetFieldValue.split(' ').length - 1].slice(0, -1) || '';

    const topPosition = targetField.offset().top + targetField.outerHeight() + 4;
    let leftPosition;
    let rightPosition;

    if (targetField.css('text-align') === 'left' ||
      (targetField.css('direction') === 'ltr' && targetField.css('text-align') === 'start')) {
      leftPosition = targetField.offset().left;
    } else {
      rightPosition = $(document.body).width() - (targetField.offset().left + targetField.outerWidth());
    }

    showAutocompleteElement(emailValue, {
      top: topPosition,
      left: leftPosition,
      right: rightPosition
    }, targetField);
  } else if (isShown) {
    hideAutocompleteElement();
  }
});

/** Paste Email **/
function insertTextAtCursor (text) { // source: stackoverflow.com/questions/2920150/
  let sel, range;
  if (window.getSelection) {
    sel = window.getSelection();
    if (sel.getRangeAt && sel.rangeCount) {
      range = sel.getRangeAt(0);
      range.deleteContents();
      range.insertNode(document.createTextNode(text));
      sel.collapseToEnd();
    }
  }
}

function insertTextAtCursorInInputs (element, text) {
  const posStart = element.selectionStart;
  const posEnd = element.selectionEnd;
  const currentValueStart = $(element).val().slice(0, posStart);
  const currentValueEnd = $(element).val().slice(posEnd);

  $(element).val(currentValueStart + text + currentValueEnd);
  $(element)[0].dispatchEvent(new Event('input', {
    bubbles: true,
    cancelable: true
  }));

  const moveCursorTo = currentValueStart.length + text.length;
  $(element)[0].setSelectionRange(moveCursorTo, moveCursorTo);
}

function insertWord (element, value) {
  if (!element || !value) {
    return;
  }

  const tag = element.tagName.toLowerCase();
  if (tag === 'input' || tag === 'textarea') {
    insertTextAtCursorInInputs(element, value);
  } else {
    insertTextAtCursor(value);
  }
}

let focusedElement;
window.addEventListener('contextmenu', (event) => {
  focusedElement = event.target;
}, false);

chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
  if (request.action === 'paste') {
    insertWord(focusedElement, request.value.menuItemId);
  }
});
